@@ -70,8 +70,6 @@ and either update state or generate new actions
 
 .. autofunction:: set_colorbar
 
-.. autofunction:: set_fixed
-
 .. autofunction:: set_reverse
 
 .. autofunction:: set_palette_name
@@ -101,10 +99,133 @@ import numpy as np
 from forest.observe import Observable
 from forest.rx import Stream
 from forest.db.util import autolabel
+from dataclasses import dataclass, asdict
 
 
+def colorbar_figure(color_mapper, plot_width=500):
+    # Dimensions
+    padding = 5
+    margin = 20
+    colorbar_height = 20
+    plot_height = colorbar_height + 30
+
+    # Colorbar
+    colorbar = bokeh.models.ColorBar(
+        color_mapper=color_mapper,
+        location=(0, 0),
+        height=colorbar_height,
+        width=int(plot_width - (margin + padding)),
+        padding=padding,
+        orientation="horizontal",
+        major_tick_line_color="black",
+        bar_line_color="black",
+        background_fill_alpha=0.,
+    )
+    colorbar.title = ""
+
+    # Figure
+    figure = bokeh.plotting.figure(
+        plot_height=plot_height,
+        plot_width=plot_width,
+        toolbar_location=None,
+        min_border=0,
+        background_fill_alpha=0,
+        border_fill_alpha=0,
+        outline_line_color=None,
+    )
+    figure.axis.visible = False
+    figure.add_layout(colorbar, 'center')
+    return figure
+
+
+
+@dataclass
+class ColorSpec:
+    """Specifies color mapper settings"""
+    name: str = "Greys"
+    number: int = 256
+    reverse: bool = False
+    low: float = 0.
+    low_visible: bool = True
+    high: float = 1.
+    high_visible: bool = True
+
+    def __post_init__(self):
+        if self.name == "Palettes":
+            self.name = "Greys"
+        try:
+            self.number = int(self.number)
+        except ValueError:
+            self.number = 256
+        self.low = float(self.low)
+        self.high = float(self.high)
+
+    @property
+    def palette(self):
+        if self.reverse:
+            step = -1
+        else:
+            step = 1
+        return bokeh.palettes.all_palettes[self.name][self.number][::step]
+
+    @property
+    def high_color(self):
+        if self.high_visible:
+            return None
+        else:
+            return bokeh.colors.RGB(0, 0, 0, a=0)
+
+    @property
+    def low_color(self):
+        if self.low_visible:
+            return None
+        else:
+            return bokeh.colors.RGB(0, 0, 0, a=0)
+
+    def apply(self, color_mapper):
+        """Helper to apply settings to color_mapper"""
+        color_mapper.palette = self.palette
+        color_mapper.low = self.low
+        color_mapper.low_color = self.low_color
+        color_mapper.high = self.high
+        color_mapper.high_color = self.high_color
+
+
+def parse_color_spec(props):
+    kwargs = {}
+
+    # Palette
+    if "name" in props:
+        kwargs["name"] = props["name"]
+    if "number" in props:
+        kwargs["number"] = props["number"]
+    if "reverse" in props:
+        kwargs["reverse"] = props["reverse"]
+    if "invisible_min" in props:
+        kwargs["low_visible"] = not props["invisible_min"]
+    if "invisible_max" in props:
+        kwargs["high_visible"] = not props["invisible_max"]
+
+    # Limits
+    origin = props.get("limits", {}).get("origin", "column_data_source")
+    attrs = props.get("limits", {}).get(origin, {})
+    if "low" in attrs:
+        try:
+            kwargs["low"] = float(attrs["low"])
+        except ValueError:
+            pass
+    if "high" in attrs:
+        try:
+            kwargs["high"] = float(attrs["high"])
+        except ValueError:
+            pass
+    return ColorSpec(**kwargs)
+
+
+SET_INVISIBLE = "SET_INVISIBLE"
 SET_PALETTE = "SET_PALETTE"
 SET_LIMITS = "SET_LIMITS"
+SET_LIMITS_ORIGIN = "SET_LIMITS_ORIGIN"
 
 
 def set_colorbar(options):
@@ -112,11 +233,6 @@ def set_colorbar(options):
     return {"kind": SET_PALETTE, "payload": options}
 
 
-def set_fixed(flag):
-    """Action to set fix user-defined limits"""
-    return {"kind": SET_PALETTE, "payload": {"fixed": flag}}
-
-
 def set_reverse(flag):
     """Action to reverse color palette colors"""
     return {"kind": SET_PALETTE, "payload": {"reverse": flag}}
@@ -148,11 +264,6 @@ def set_source_limits(low, high):
             "payload": {"low": low, "high": high},
             "meta": {"origin": "column_data_source"}}
 
-def is_source_origin(action):
-    """Detect origin of set_limits action"""
-    origin = action.get("meta", {}).get("origin", "")
-    return origin == "column_data_source"
-
 
 def set_user_high(high):
     """Action to set user defined colorbar higher limit"""
@@ -168,13 +279,18 @@ def set_user_low(low):
             "meta": {"origin": "user"}}
 
 
+def set_limits_origin(text):
+    """Action to set limits origin, e.g. user/column_data_source"""
+    return {"kind": SET_LIMITS_ORIGIN, "payload": text}
+
+
 def set_invisible_min(flag):
     """Action to mask out data below colour bar limits"""
-    return {"kind": SET_LIMITS, "payload": {"invisible_min": flag}}
+    return {"kind": SET_INVISIBLE, "payload": {"invisible_min": flag}}
 
 def set_invisible_max(flag):
     """Action to mask out data below colour bar limits"""
-    return {"kind": SET_LIMITS, "payload": {"invisible_max": flag}}
+    return {"kind": SET_INVISIBLE, "payload": {"invisible_max": flag}}
 
 
 def reducer(state, action):
@@ -188,12 +304,41 @@ def reducer(state, action):
     """
     state = copy.deepcopy(state)
     kind = action["kind"]
-    if kind in [SET_PALETTE, SET_LIMITS]:
+    if kind in [SET_PALETTE, SET_INVISIBLE]:
         state["colorbar"] = state.get("colorbar", {})
         state["colorbar"].update(action["payload"])
     return state
 
 
+def limits_reducer(state, action):
+    state = copy.deepcopy(state)
+
+    if action["kind"] == SET_LIMITS_ORIGIN:
+        # Build/traverse tree
+        node = state
+        keys = ("colorbar", "limits")
+        for key in keys:
+            node[key] = node.get(key, {})
+            node = node[key]
+        node.update({"origin": action["payload"]})
+
+    elif meta_origin(action) in {"user", "column_data_source"}:
+        # Build/traverse tree
+        node = state
+        keys = ("colorbar", "limits", meta_origin(action))
+        for key in keys:
+            node[key] = node.get(key, {})
+            node = node[key]
+        node.update(action["payload"])
+        return state
+
+    return state
+
+
+def meta_origin(action):
+    return action.get("meta", {}).get("origin", "")
+
+
 def defaults():
     """Default color palette settings
 
@@ -206,7 +351,6 @@ def defaults():
             "numbers": palette_numbers("Viridis"),
             "low": 0,
             "high": 1,
-            "fixed": False,
             "reverse": False,
             "invisible_min": False,
             "invisible_max": False,
@@ -224,7 +368,6 @@ def defaults():
         "numbers": palette_numbers("Viridis"),
         "low": 0,
         "high": 1,
-        "fixed": False,
         "reverse": False,
         "invisible_min": False,
         "invisible_max": False,
@@ -258,9 +401,6 @@ def palettes(store, action):
     .. note:: middleware is an action generator
     """
     kind = action["kind"]
-    if (kind == SET_LIMITS) and is_fixed(store.state) and is_source_origin(action):
-        # Filter SET_LIMIT actions from ColumnDataSource
-        return
     if kind == SET_PALETTE:
         payload = action["payload"]
         if "name" in payload:
@@ -298,11 +438,6 @@ def middleware():
     return call
 
 
-def is_fixed(state):
-    """Helper to discover if fixed limits have been selected"""
-    return state.get("colorbar", {}).get("fixed", False)
-
-
 def palette_numbers(name):
     """Helper to choose available color palette numbers
 
@@ -319,49 +454,88 @@ class SourceLimits(Observable):
     of connecting to a :class:`forest.redux.Store`, simply
     subscribe ``store.dispatch`` to action events.
 
-    >>> source_limits = SourceLimits(sources)
-    >>> source_limits.add_subscriber(store.dispatch)
+    >>> source_limits = SourceLimits()
+    >>> for source in sources:
+    ...     source_limits.add_source(source)
+    >>> source_limits.connect(store)
 
     .. note:: Unlike a typical component there is no ``layout`` property
               to attach to a bokeh document
     """
-    def __init__(self, sources):
-        self.sources = sources
-        for source in self.sources:
-            source.on_change("data", self.on_change)
+    def __init__(self):
+        self.sources = []
         super().__init__()
 
+    def connect(self, store):
+        """Connect events to the Store"""
+        self.add_subscriber(store.dispatch)
+        return self
+
+    def add_source(self, source):
+        """Add ColumnDataSource to listened sources"""
+        if source not in self.sources:
+            source.on_change("data", self.on_change)
+            self.sources.append(source)
+
+    def remove_source(self, source):
+        """Remove ColumnDataSource from listened sources"""
+        # Remove on_change handler
+        try:
+            source.remove_on_change("data", self.on_change)
+        except ValueError:
+            pass
+
+        # Remove source from limit calculation
+        if source in self.sources:
+            self.sources = [_source for _source in self.sources
+                            if _source.id != source.id]
+            low, high = self.limits(self.sources)
+            self.notify(set_source_limits(low, high))
+
     def on_change(self, attr, old, new):
+        """Generate action from bokeh event"""
+        low, high = self.limits(self.sources)
+        self.notify(set_source_limits(low, high))
+
+    def limits(self, sources):
+        """Calculate limits from underlying sources"""
         images = []
-        for source in self.sources:
+        for source in sources:
             if len(source.data["image"]) == 0:
                 continue
             images.append(source.data["image"][0])
         if len(images) > 0:
             low = np.min([np.min(x) for x in images])
             high = np.max([np.max(x) for x in images])
-            self.notify(set_source_limits(low, high))
+            return low, high
         else:
-            self.notify(set_source_limits(0, 1))
+            return 0, 1
 
 
 class UserLimits(Observable):
     """User controlled color mapper limits"""
     def __init__(self):
         self.inputs = {
-            "low": bokeh.models.TextInput(title="Min:"),
-            "high": bokeh.models.TextInput(title="Max:")
+            "low": bokeh.models.TextInput(title="User min:",
+                                          placeholder="Enter a number"),
+            "high": bokeh.models.TextInput(title="User max:",
+                                           placeholder="Enter a number"),
+            "source_low": bokeh.models.TextInput(title="Data min:",
+                                                 disabled=True),
+            "source_high": bokeh.models.TextInput(title="Data max:",
+                                                  disabled=True)
         }
         self.inputs["low"].on_change("value", self.on_input_low)
         self.inputs["high"].on_change("value", self.on_input_high)
 
-        self.checkboxes = {}
+        # RadioGroup for user/data limits
+        self.radio_group = bokeh.models.RadioGroup(
+            labels=["Use data limits", "Use user limits"],
+            active=0,
+            inline=True)
+        self.radio_group.on_change("active", self.on_origin)
 
-        # Checkbox fix data limits to user supplied limits
-        self.checkboxes["fixed"] = bokeh.models.CheckboxGroup(
-                labels=["Fix min/max settings for all frames"],
-                active=[])
-        self.checkboxes["fixed"].on_change("active", self.on_checkbox_change)
+        self.checkboxes = {}
 
         # Checkbox transparency lower threshold
         self.checkboxes["invisible_min"] = bokeh.models.CheckboxGroup(
@@ -375,10 +549,21 @@ class UserLimits(Observable):
             active=[])
         self.checkboxes["invisible_max"].on_change("active", self.on_invisible_max)
 
+        widths = {
+            "row": 310
+        }
         self.layout = bokeh.layouts.column(
-            self.inputs["low"],
-            self.inputs["high"],
-            self.checkboxes["fixed"],
+            bokeh.layouts.row(
+                self.inputs["low"],
+                self.inputs["high"],
+                width=widths["row"]),
+            bokeh.layouts.row(
+                self.inputs["source_low"],
+                self.inputs["source_high"],
+                width=widths["row"]),
+            bokeh.layouts.row(
+                self.radio_group,
+                width=widths["row"]),
             self.checkboxes["invisible_min"],
             self.checkboxes["invisible_max"],
         )
@@ -396,14 +581,13 @@ class UserLimits(Observable):
         connect(self, store)
         return self
 
-    def on_checkbox_change(self, attr, old, new):
-        self.notify(set_fixed(len(new) == 1))
-
     def on_input_low(self, attr, old, new):
-        self.notify(set_user_low(float(new)))
+        """Event-handler to set user low"""
+        self.notify(set_user_low(new))
 
     def on_input_high(self, attr, old, new):
-        self.notify(set_user_high(float(new)))
+        """Event-handler to set user high"""
+        self.notify(set_user_high(new))
 
     def on_invisible_min(self, attr, old, new):
         """Event-handler when invisible_min toggle is changed"""
@@ -413,18 +597,70 @@ class UserLimits(Observable):
         """Event-handler when invisible_max toggle is changed"""
         self.notify(set_invisible_max(len(new) == 1))
 
+    def on_origin(self, attr, old, new):
+        origin = {1: "user"}.get(new, "column_data_source")
+        self.notify(set_limits_origin(origin))
+
+    def props(self):
+        """Helper to get current state of widgets"""
+        _props = {
+            "limits": {
+                "origin": {
+                    0: "column_data_source",
+                    1: "user"
+                }[self.radio_group.active],
+                "user": {},
+                "column_data_source": {},
+            }
+        }
+
+        # User inputs
+        if self.inputs["high"].value is not None:
+            _props["limits"]["user"]["high"] = self.inputs["high"].value
+        if self.inputs["low"].value is not None:
+            _props["limits"]["user"]["low"] = self.inputs["low"].value
+
+        # ColumnDataSource inputs
+        if self.inputs["source_high"].value is not None:
+            _props["limits"]["column_data_source"]["high"] = self.inputs["source_high"].value
+        if self.inputs["source_low"].value is not None:
+            _props["limits"]["column_data_source"]["low"] = self.inputs["source_low"].value
+
+        # Invisible min/max
+        for key in ("invisible_min", "invisible_max"):
+            _props[key] = len(self.checkboxes[key].active) == 1
+        return _props
+
     def render(self, props):
         """Update user-defined limits inputs"""
-        for key in ["fixed", "invisible_min", "invisible_max"]:
+        for key in ["invisible_min", "invisible_max"]:
             if props.get(key, False):
                 self.checkboxes[key].active = [0]
             else:
                 self.checkboxes[key].active = []
 
-        if "high" in props:
-            self.inputs["high"].value = str(props["high"])
-        if "low" in props:
-            self.inputs["low"].value = str(props["low"])
+        # User limits
+        origin = "user"
+        attrs = props.get("limits", {}).get(origin, {})
+        if "high" in attrs:
+            self.inputs["high"].value = str(attrs["high"])
+        if "low" in attrs:
+            self.inputs["low"].value = str(attrs["low"])
+
+        # ColumnDataSource limits
+        origin = "column_data_source"
+        attrs = props.get("limits", {}).get(origin, {})
+        if "high" in attrs:
+            self.inputs["source_high"].value = str(attrs["high"])
+        if "low" in attrs:
+            self.inputs["source_low"].value = str(attrs["low"])
+
+        # Sync radio group
+        origin = props.get("limits", {}).get("origin", "column_data_source")
+        if origin == "column_data_source":
+            self.radio_group.active = 0
+        else:
+            self.radio_group.active = 1
 
 
 def state_to_props(state):
@@ -450,6 +686,10 @@ def connect(view, store):
     props and filters out duplicates.
     """
     view.add_subscriber(store.dispatch)
+    one_way_connect(view, store)
+
+
+def one_way_connect(view, store):
     stream = (Stream()
                 .listen_to(store)
                 .map(state_to_props)
@@ -458,10 +698,125 @@ def connect(view, store):
     stream.map(lambda props: view.render(props))
 
 
-class ColorPalette(Observable):
-    """Color palette user interface"""
+class ColorMapperView:
     def __init__(self, color_mapper):
         self.color_mapper = color_mapper
+
+    def connect(self, store):
+        """Connect component to Store"""
+        one_way_connect(self, store)
+        return self
+
+    def render(self, props):
+        if isinstance(props, ColorSpec):
+            spec = props
+        else:
+            spec = parse_color_spec(props)
+        spec.apply(self.color_mapper)
+        return
+
+
+class ColorPaletteJS:
+    """Client-side ColorPalette selector"""
+    def __init__(self):
+        self.widths = {
+            "select": 140,
+            "div": 300
+        }
+        # Map palettes to ColumnDataSource
+        names, numbers = [], []
+        for name, palettes in sorted(bokeh.palettes.all_palettes.items()):
+            for number in sorted(palettes.keys()):
+                names.append(name)
+                numbers.append(number)
+        self.source = bokeh.models.ColumnDataSource({
+            "names": names,
+            "numbers": numbers
+        })
+
+        # Figure to display color bar preview
+        self.color_mapper = bokeh.models.LinearColorMapper(
+            palette="Greys256",
+            low=0,
+            high=1)
+        self.figure = colorbar_figure(self.color_mapper,
+                                      plot_width=320)
+
+        # Wire up select widgets
+        self.selects = {
+            "name": bokeh.models.Select(width=self.widths["select"]),
+            "number": bokeh.models.Select(width=self.widths["select"]),
+        }
+        self.selects["name"].options = ["Please specify"] + list(sorted(set(names)))
+        self.selects["name"].value = "Please specify"
+        self.selects["number"].options = ["Please specify"]
+        self.selects["number"].value = "Please specify"
+        custom_js = bokeh.models.CustomJS(args=dict(
+                source=self.source,
+                select=self.selects["number"]), code="""
+            let name = cb_obj.value
+            let names = source.data["names"]
+            let numbers = source.data["numbers"]
+            let options = ["Please specify"]
+            for (let i=0; i<names.length; i++) {
+                if (names[i] == name) {
+                    options.push(numbers[i].toString())
+                }
+            }
+            select.options = options
+        """)
+        self.selects["name"].js_on_change("value", custom_js)
+
+
+        # Preview figure
+        self.selects["name"].on_change("value", self.on_preview)
+        self.selects["number"].on_change("value", self.on_preview)
+
+        # Reverse checkbox
+        self.checkboxes = {}
+        self.checkboxes["reverse"] = bokeh.models.CheckboxGroup(
+            labels=["Reverse"],
+            active=[])
+        self.checkboxes["reverse"].on_change("active", self.on_preview)
+
+        self.layout = bokeh.layouts.column(
+            bokeh.models.Div(text="Color palette:",
+                             width=self.widths["div"]),
+            self.figure,
+            bokeh.layouts.row(
+                self.selects["name"],
+                self.selects["number"]),
+            self.checkboxes["reverse"])
+
+    def on_preview(self, attr, old, new):
+        spec = ColorSpec(**self.props())
+        try:
+            spec.apply(self.color_mapper)
+        except KeyError:
+            pass
+
+    def props(self):
+        """Useful for aggregating form data"""
+        _props = {}
+        for key, select in self.selects.items():
+            if select.value is not None:
+                _props[key] = select.value
+        _props["reverse"] = len(self.checkboxes["reverse"].active) == 1
+        return _props
+
+    def render(self, props):
+        for key, select in self.selects.items():
+            if key in props:
+                select.value = str(props[key])
+        self.checkboxes["reverse"].active = {
+            True: [0],
+            False: []
+        }[props.get("reverse", False)]
+
+
+class ColorPalette(Observable):
+    """Color palette user interface"""
+    def __init__(self):
         self.dropdowns = {
             "names": bokeh.models.Dropdown(label="Palettes"),
             "numbers": bokeh.models.Dropdown(label="N")
@@ -486,6 +841,14 @@ class ColorPalette(Observable):
         connect(self, store)
         return self
 
+    def props(self):
+        """Helper to get widget settings"""
+        return {
+            "name": self.dropdowns["names"].label,
+            "number": self.dropdowns["numbers"].label,
+            "reverse": len(self.checkbox.active) == 1
+        }
+
     def on_name(self, attr, old, new):
         """Event-handler when a palette name is selected"""
         self.notify(set_palette_name(new))
@@ -501,47 +864,22 @@ class ColorPalette(Observable):
     def render(self, props):
         """Render component from properties derived from state"""
         assert isinstance(props, dict), "only support dict"
+
+        spec = parse_color_spec(props)
         if "name" in props:
-            self.dropdowns["names"].label = props["name"]
+            self.dropdowns["names"].label = spec.name
         if "number" in props:
-            self.dropdowns["numbers"].label = str(props["number"])
-        if ("name" in props) and ("number" in props):
-            name = props["name"]
-            number = props["number"]
-            reverse = props.get("reverse", False)
-            palette = self.palette(name, number)
-            if reverse:
-                palette = palette[::-1]
-            self.color_mapper.palette = palette
+            self.dropdowns["numbers"].label = str(spec.number)
+
         if "names" in props:
             values = props["names"]
             self.dropdowns["names"].menu = list(zip(values, values))
         if "numbers" in props:
             values = [str(n) for n in props["numbers"]]
             self.dropdowns["numbers"].menu = list(zip(values, values))
-        if "low" in props:
-            self.color_mapper.low = props["low"]
-        if "high" in props:
-            self.color_mapper.high = props["high"]
-        invisible_min = props.get("invisible_min", False)
-        if invisible_min:
-            color = bokeh.colors.RGB(0, 0, 0, a=0)
-            self.color_mapper.low_color = color
-        else:
-            self.color_mapper.low_color = None
-        invisible_max = props.get("invisible_max", False)
-        if invisible_max:
-            color = bokeh.colors.RGB(0, 0, 0, a=0)
-            self.color_mapper.high_color = color
-        else:
-            self.color_mapper.high_color = None
 
-        # Render reverse checkbox state
-        if props.get("reverse", False):
+        # Render checkbox state
+        if spec.reverse:
             self.checkbox.active = [0]
         else:
             self.checkbox.active = []
-
-    @staticmethod
-    def palette(name, number):
-        return bokeh.palettes.all_palettes[name][number]
